==> equations-3.1.2.tar.gz: example/flutter_example/test/routes/polynomial_page/polynomial_data_input_test.dart <==
import 'package:equations_solver/blocs/polynomial_solver/polynomial_solver.dart';
import 'package:equations_solver/blocs/precision_slider/precision_slider.dart';
import 'package:equations_solver/routes/polynomial_page/polynomial_data_input.dart';
import 'package:equations_solver/routes/polynomial_page/polynomial_input_field.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:golden_toolkit/golden_toolkit.dart';
import 'package:mocktail/mocktail.dart';

import '../../utils/bloc_mocks.dart';
import '../mock_wrapper.dart';

void main() {
  late final List<BlocProvider> providers;
  late final PolynomialBloc polynomialBloc;

  setUpAll(() {
    registerFallbackValue(MockPolynomialEvent());
    registerFallbackValue(MockPolynomialState());

    polynomialBloc = MockPolynomialBloc();

    providers = [
      BlocProvider<PolynomialBloc>.value(
        value: polynomialBloc,
      ),
      BlocProvider<PrecisionSliderCubit>(
        create: (_) => PrecisionSliderCubit(
          minValue: 1,
          maxValue: 10,
        ),
      ),
    ];
  });

  group("Testing the 'PolynomialDataInput' widget", () {
    testWidgets(
      "Making sure that with a 'linear' configuration type only "
      '2 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.linear);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(2));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      "Making sure that with a 'quadratic' configuration type only "
      '3 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.quadratic);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(3));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      "Making sure that with a 'cubic' configuration type only "
      '4 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.cubic);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(4));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      "Making sure that with a 'cubic' configuration type only "
      '5 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.quartic);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(5));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      'Making sure that when trying to solve an equation, if at '
      'least one of the inputs is wrong, a snackbar appears',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.linear);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        // No snackbar by default
        expect(find.byType(SnackBar), findsNothing);

        // Tap the 'Solve' button
        final finder = find.byKey(const Key('Polynomial-button-solve'));
        await tester.tap(finder);

        // The snackbar appeared
        await tester.pumpAndSettle();
        expect(find.byType(SnackBar), findsOneWidget);
      },
    );

    testWidgets('Making sure that equations can be solved', (tester) async {
      final bloc = PolynomialBloc(PolynomialType.quadratic);

      await tester.pumpWidget(MockWrapper(
        child: MultiBlocProvider(
          providers: providers,
          child: Scaffold(
            body: BlocProvider<PolynomialBloc>.value(
              value: bloc,
              child: const PolynomialDataInput(),
            ),
          ),
        ),
      ));

      final coeffA = find.byKey(
        const Key('PolynomialInputField-coefficient-0'),
      );
      final coeffB = find.byKey(
        const Key('PolynomialInputField-coefficient-1'),
      );
      final coeffC = find.byKey(
        const Key('PolynomialInputField-coefficient-2'),
      );
      final solveButton = find.byKey(
        const Key('Polynomial-button-solve'),
      );
      final cleanButton = find.byKey(
        const Key('Polynomial-button-clean'),
      );

      // Filling the forms
      await tester.enterText(coeffA, '-5');
      await tester.enterText(coeffB, '1/2');
      await tester.enterText(coeffC, '3');

      // Making sure that there are no results
      expect(bloc.state, isA<PolynomialNone>());

      // Solving the equation
      await tester.tap(solveButton);
      await tester.pumpAndSettle();

      // Sending it to the bloc
      expect(bloc.state, isA<PolynomialRoots>());

      // Cleaning the UI
      await tester.tap(cleanButton);
      await tester.pumpAndSettle();

      expect(bloc.state, isA<PolynomialNone>());
      tester
          .widgetList<TextFormField>(find.byType(TextFormField))
          .forEach((input) {
        expect(input.controller!.text.length, isZero);
      });
    });

    testGoldens('PolynomialDataInput - Linear equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.linear);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Linear equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_linear');
    });

    testGoldens('PolynomialDataInput - Quadratic equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.quadratic);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Quadratic equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_quadratic');
    });

    testGoldens('PolynomialDataInput - Cubic equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.cubic);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Cubic equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_cubic');
    });

    testGoldens('PolynomialDataInput - Quartic equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.quartic);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Quartic equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_quartic');
    });
  });
}

==> equations-3.1.3.tar.gz: example/flutter_example/test/routes/polynomial_page/polynomial_data_input_test.dart <==
import 'package:equations_solver/blocs/polynomial_solver/polynomial_solver.dart';
import 'package:equations_solver/blocs/precision_slider/precision_slider.dart';
import 'package:equations_solver/routes/polynomial_page/polynomial_data_input.dart';
import 'package:equations_solver/routes/polynomial_page/polynomial_input_field.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:golden_toolkit/golden_toolkit.dart';
import 'package:mocktail/mocktail.dart';

import '../../utils/bloc_mocks.dart';
import '../mock_wrapper.dart';

void main() {
  late final List<BlocProvider> providers;
  late final PolynomialBloc polynomialBloc;

  setUpAll(() {
    registerFallbackValue(MockPolynomialEvent());
    registerFallbackValue(MockPolynomialState());

    polynomialBloc = MockPolynomialBloc();

    providers = [
      BlocProvider<PolynomialBloc>.value(
        value: polynomialBloc,
      ),
      BlocProvider<PrecisionSliderCubit>(
        create: (_) => PrecisionSliderCubit(
          minValue: 1,
          maxValue: 10,
        ),
      ),
    ];
  });

  group("Testing the 'PolynomialDataInput' widget", () {
    testWidgets(
      "Making sure that with a 'linear' configuration type only "
      '2 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.linear);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(2));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      "Making sure that with a 'quadratic' configuration type only "
      '3 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.quadratic);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(3));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      "Making sure that with a 'cubic' configuration type only "
      '4 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.cubic);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(4));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      "Making sure that with a 'cubic' configuration type only "
      '5 input fields appear on the screen',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.quartic);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        expect(find.byType(PolynomialDataInput), findsOneWidget);
        expect(find.byType(PolynomialInputField), findsNWidgets(5));

        // To make sure that fields validation actually happens
        expect(find.byType(Form), findsOneWidget);
      },
    );

    testWidgets(
      'Making sure that when trying to solve an equation, if at '
      'least one of the inputs is wrong, a snackbar appears',
      (tester) async {
        when(() => polynomialBloc.polynomialType)
            .thenReturn(PolynomialType.linear);

        await tester.pumpWidget(MockWrapper(
          child: MultiBlocProvider(
            providers: providers,
            child: const Scaffold(
              body: PolynomialDataInput(),
            ),
          ),
        ));

        // No snackbar by default
        expect(find.byType(SnackBar), findsNothing);

        // Tap the 'Solve' button
        final finder = find.byKey(const Key('Polynomial-button-solve'));
        await tester.tap(finder);

        // The snackbar appeared
        await tester.pumpAndSettle();
        expect(find.byType(SnackBar), findsOneWidget);
      },
    );

    testWidgets('Making sure that equations can be solved', (tester) async {
      late FocusScopeNode focusScope;
      final bloc = PolynomialBloc(PolynomialType.quadratic);

      await tester.pumpWidget(MockWrapper(
        child: MultiBlocProvider(
          providers: providers,
          child: Scaffold(
            body: BlocProvider<PolynomialBloc>.value(
              value: bloc,
              child: Builder(
                builder: (context) {
                  focusScope = FocusScope.of(context);

                  return const PolynomialDataInput();
                },
              ),
            ),
          ),
        ),
      ));

      final coeffA = find.byKey(
        const Key('PolynomialInputField-coefficient-0'),
      );
      final coeffB = find.byKey(
        const Key('PolynomialInputField-coefficient-1'),
      );
      final coeffC = find.byKey(
        const Key('PolynomialInputField-coefficient-2'),
      );
      final solveButton = find.byKey(
        const Key('Polynomial-button-solve'),
      );
      final cleanButton = find.byKey(
        const Key('Polynomial-button-clean'),
      );

      // Filling the forms
      await tester.enterText(coeffA, '-5');
      await tester.enterText(coeffB, '1/2');
      await tester.enterText(coeffC, '3');

      expect(bloc.state, isA<PolynomialNone>());
      expect(focusScope.hasFocus, isTrue);

      // Solving the equation
      await tester.tap(solveButton);
      await tester.pumpAndSettle();

      // Sending it to the bloc
      expect(bloc.state, isA<PolynomialRoots>());

      // Cleaning the UI
      await tester.tap(cleanButton);
      await tester.pumpAndSettle();

      expect(bloc.state, isA<PolynomialNone>());
      expect(focusScope.hasFocus, isFalse);
      tester.widgetList<TextFormField>(find.byType(TextFormField)).forEach((t) {
        expect(t.controller!.text.length, isZero);
      });
    });

    testGoldens('PolynomialDataInput - Linear equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.linear);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Linear equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_linear');
    });

    testGoldens('PolynomialDataInput - Quadratic equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.quadratic);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Quadratic equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_quadratic');
    });

    testGoldens('PolynomialDataInput - Cubic equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.cubic);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Cubic equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_cubic');
    });

    testGoldens('PolynomialDataInput - Quartic equation', (tester) async {
      when(() => polynomialBloc.polynomialType)
          .thenReturn(PolynomialType.quartic);

      final builder = GoldenBuilder.column()
        ..addScenario(
          'Quartic equations',
          SizedBox(
            width: 500,
            height: 400,
            child: MultiBlocProvider(
              providers: providers,
              child: const PolynomialDataInput(),
            ),
          ),
        );

      await tester.pumpWidgetBuilder(
        builder.build(),
        wrapper: (child) => MockWrapper(
          child: child,
        ),
        surfaceSize: const Size(500, 500),
      );

      await screenMatchesGolden(tester, 'polynomial_input_quartic');
    });
  });
}
